@@ -1,12 +1,13 @@
-from ._core import Injectable, Module
-from ._core import Mode as InjectableMode
-from ._core import Priority as ModulePriority
+from ._core.module import Injectable, Module
+from ._core.module import Mode as InjectableMode
+from ._core.module import Priority as ModulePriority
 
 __all__ = (
     "Injectable",
     "InjectableMode",
     "Module",
     "ModulePriority",
+    "constant",
     "find_instance",
     "get_instance",
     "get_lazy_instance",
@@ -19,13 +20,14 @@ __all__ = (
 )
 
 
-def mod(name: str = None, /) -> Module:
+def mod(name: str | None = None, /) -> Module:
     if name is None:
         return Module.default()
 
     return Module.from_name(name)
 
 
+constant = mod().constant
 find_instance = mod().find_instance
 get_instance = mod().get_instance
 get_lazy_instance = mod().get_lazy_instance
