@@ -18,7 +18,7 @@ from functools import partialmethod, singledispatchmethod, update_wrapper
 from inspect import Signature, isclass
 from logging import Logger, getLogger
 from queue import Empty, Queue
-from types import MethodType, UnionType
+from types import MethodType
 from typing import (
     Any,
     ClassVar,
@@ -28,6 +28,7 @@ from typing import (
     NoReturn,
     Protocol,
     Self,
+    override,
     runtime_checkable,
 )
 from uuid import uuid4
@@ -37,8 +38,9 @@ from injection._core.common.invertible import Invertible, SimpleInvertible
 from injection._core.common.lazy import Lazy, LazyMapping
 from injection._core.common.threading import synchronized
 from injection._core.common.type import (
+    InputType,
+    TypeDef,
     TypeInfo,
-    TypeReport,
     analyze_types,
     get_return_types,
 )
@@ -50,16 +52,6 @@ from injection.exceptions import (
     NoInjectable,
 )
 
-__all__ = (
-    "Injectable",
-    "InjectableFactory",
-    "Mode",
-    "ModeStr",
-    "Module",
-    "Priority",
-    "PriorityStr",
-)
-
 """
 Events
 """
@@ -71,13 +63,14 @@ class LocatorEvent(Event, ABC):
 
 
 @dataclass(frozen=True, slots=True)
-class LocatorDependenciesUpdated(LocatorEvent):
-    reports: Collection[TypeReport]
+class LocatorDependenciesUpdated[T](LocatorEvent):
+    classes: Collection[TypeDef[T]]
     mode: Mode
 
+    @override
     def __str__(self) -> str:
-        length = len(self.reports)
-        formatted_types = ", ".join(f"`{report.type}`" for report in self.reports)
+        length = len(self.classes)
+        formatted_types = ", ".join(f"`{cls}`" for cls in self.classes)
         return (
             f"{length} dependenc{"ies" if length > 1 else "y"} have been "
             f"updated{f": {formatted_types}" if formatted_types else ""}."
@@ -93,6 +86,7 @@ class ModuleEvent(Event, ABC):
 class ModuleEventProxy(ModuleEvent):
     event: Event
 
+    @override
     def __str__(self) -> str:
         return f"`{self.module}` has propagated an event: {self.origin}"
 
@@ -113,6 +107,7 @@ class ModuleAdded(ModuleEvent):
     module_added: Module
     priority: Priority
 
+    @override
     def __str__(self) -> str:
         return f"`{self.module}` now uses `{self.module_added}`."
 
@@ -121,6 +116,7 @@ class ModuleAdded(ModuleEvent):
 class ModuleRemoved(ModuleEvent):
     module_removed: Module
 
+    @override
     def __str__(self) -> str:
         return f"`{self.module}` no longer uses `{self.module_removed}`."
 
@@ -130,6 +126,7 @@ class ModulePriorityUpdated(ModuleEvent):
     module_updated: Module
     priority: Priority
 
+    @override
     def __str__(self) -> str:
         return (
             f"In `{self.module}`, the priority `{self.priority}` "
@@ -150,7 +147,7 @@ class Injectable[T](Protocol):
     def is_locked(self) -> bool:
         return False
 
-    def unlock(self):
+    def unlock(self) -> None:
         return
 
     @abstractmethod
@@ -163,9 +160,10 @@ class BaseInjectable[T](Injectable[T], ABC):
     factory: Callable[..., T]
 
 
-class NewInjectable[T](BaseInjectable[T]):
+class SimpleInjectable[T](BaseInjectable[T]):
     __slots__ = ()
 
+    @override
     def get_instance(self) -> T:
         return self.factory()
 
@@ -180,12 +178,15 @@ class SingletonInjectable[T](BaseInjectable[T]):
         return self.__dict__
 
     @property
+    @override
     def is_locked(self) -> bool:
         return self.__key in self.cache
 
-    def unlock(self):
+    @override
+    def unlock(self) -> None:
         self.cache.clear()
 
+    @override
     def get_instance(self) -> T:
         with suppress(KeyError):
             return self.cache[self.__key]
@@ -201,6 +202,7 @@ class SingletonInjectable[T](BaseInjectable[T]):
 class ShouldBeInjectable[T](Injectable[T]):
     cls: type[T]
 
+    @override
     def get_instance(self) -> NoReturn:
         raise InjectionError(f"`{self.cls}` should be an injectable.")
 
@@ -215,11 +217,11 @@ class Broker(Protocol):
     __slots__ = ()
 
     @abstractmethod
-    def __getitem__[T](self, cls: type[T] | UnionType, /) -> Injectable[T]:
+    def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
         raise NotImplementedError
 
     @abstractmethod
-    def __contains__(self, cls: type | UnionType, /) -> bool:
+    def __contains__(self, cls: InputType[Any], /) -> bool:
         raise NotImplementedError
 
     @property
@@ -261,45 +263,47 @@ class Record[T](NamedTuple):
 
 @dataclass(repr=False, frozen=True, slots=True)
 class Locator(Broker):
-    __records: dict[TypeReport, Record] = field(default_factory=dict, init=False)
+    __records: dict[TypeDef[Any], Record[Any]] = field(default_factory=dict, init=False)
     __channel: EventChannel = field(default_factory=EventChannel, init=False)
 
-    def __getitem__[T](self, cls: type[T] | UnionType, /) -> Injectable[T]:
-        for report in analyze_types(cls):
-            for scoped_report in OrderedDict.fromkeys((report, report.no_args)):
-                try:
-                    injectable, _ = self.__records[scoped_report]
-                except KeyError:
-                    continue
+    @override
+    def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
+        for analyzed_class in analyze_types(cls, with_origin=True):
+            try:
+                record = self.__records[analyzed_class]
+            except KeyError:
+                continue
 
-                return injectable
+            return record.injectable
 
         raise NoInjectable(cls)
 
-    def __contains__(self, cls: type | UnionType, /) -> bool:
-        return any(report in self.__records for report in analyze_types(cls))
+    @override
+    def __contains__(self, cls: InputType[Any], /) -> bool:
+        return any(
+            analyzed_class in self.__records
+            for analyzed_class in analyze_types(cls, with_origin=True)
+        )
 
     @property
+    @override
     def is_locked(self) -> bool:
         return any(injectable.is_locked for injectable in self.__injectables)
 
     @property
-    def __injectables(self) -> frozenset[Injectable]:
+    def __injectables(self) -> frozenset[Injectable[Any]]:
         return frozenset(injectable for injectable, _ in self.__records.values())
 
     @synchronized()
     def update[T](
         self,
-        classes: Iterable[type[T] | UnionType],
+        classes: Iterable[InputType[T]],
         injectable: Injectable[T],
         mode: Mode | ModeStr,
     ) -> Self:
         mode = Mode(mode)
         record = Record(injectable, mode)
-        records = {
-            report: record
-            for report in self.__prepare_reports_for_updating(classes, mode)
-        }
+        records = {cls: record for cls in self.__prepare_for_updating(classes, mode)}
 
         if records:
             event = LocatorDependenciesUpdated(self, records.keys(), mode)
@@ -309,6 +313,7 @@ class Locator(Broker):
 
         return self
 
+    @override
     @synchronized()
     def unlock(self) -> Self:
         for injectable in self.__injectables:
@@ -320,33 +325,35 @@ class Locator(Broker):
         self.__channel.add_listener(listener)
         return self
 
-    def dispatch(self, event: Event) -> ContextManager:
+    def dispatch(self, event: Event) -> ContextManager[None]:
         return self.__channel.dispatch(event)
 
-    def __prepare_reports_for_updating(
+    def __prepare_for_updating[T](
         self,
-        classes: Iterable[type | UnionType],
+        classes: Iterable[InputType[T]],
         mode: Mode,
-    ) -> Iterator[TypeReport]:
+    ) -> Iterator[TypeDef[T]]:
         rank = mode.rank
 
-        for report in frozenset(analyze_types(*classes)):
+        for cls in frozenset(analyze_types(*classes)):
             try:
-                _, current_mode = self.__records[report]
+                record = self.__records[cls]
 
             except KeyError:
-                pass
+                ...
 
             else:
+                current_mode = record.mode
+
                 if mode == current_mode and mode != Mode.OVERRIDE:
                     raise RuntimeError(
-                        f"An injectable already exists for the class `{report.type}`."
+                        f"An injectable already exists for the class `{cls}`."
                     )
 
                 elif rank < current_mode.rank:
                     continue
 
-            yield report
+            yield cls
 
 
 """
@@ -382,7 +389,7 @@ class Module(Broker, EventListener):
         repr=False,
     )
     __loggers: list[Logger] = field(
-        default_factory=lambda: [getLogger(__name__)],
+        default_factory=lambda: [getLogger("python-injection")],
         init=False,
         repr=False,
     )
@@ -394,20 +401,23 @@ class Module(Broker, EventListener):
 
     __instances: ClassVar[dict[str, Module]] = {}
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         self.__locator.add_listener(self)
 
-    def __getitem__[T](self, cls: type[T] | UnionType, /) -> Injectable[T]:
+    @override
+    def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
         for broker in self.__brokers:
             with suppress(KeyError):
                 return broker[cls]
 
         raise NoInjectable(cls)
 
-    def __contains__(self, cls: type | UnionType, /) -> bool:
+    @override
+    def __contains__(self, cls: InputType[Any], /) -> bool:
         return any(cls in broker for broker in self.__brokers)
 
     @property
+    @override
     def is_locked(self) -> bool:
         return any(broker.is_locked for broker in self.__brokers)
 
@@ -416,17 +426,17 @@ class Module(Broker, EventListener):
         yield from tuple(self.__modules)
         yield self.__locator
 
-    def injectable[T](
+    def injectable[**P, T](  # type: ignore[no-untyped-def]
         self,
-        wrapped: Callable[..., T] = None,
+        wrapped: Callable[P, T] | None = None,
         /,
         *,
-        cls: InjectableFactory[T] = NewInjectable,
+        cls: InjectableFactory[T] = SimpleInjectable,
         inject: bool = True,
         on: TypeInfo[T] = (),
         mode: Mode | ModeStr = Mode.get_default(),
     ):
-        def decorator(wp):
+        def decorator(wp):  # type: ignore[no-untyped-def]
             factory = self.inject(wp, return_factory=True) if inject else wp
             injectable = cls(factory)
             classes = get_return_types(wp, on)
@@ -437,8 +447,8 @@ class Module(Broker, EventListener):
 
     singleton = partialmethod(injectable, cls=SingletonInjectable)
 
-    def should_be_injectable(self, wrapped: type = None, /):
-        def decorator(wp):
+    def should_be_injectable[T](self, wrapped: type[T] | None = None, /):  # type: ignore[no-untyped-def]
+        def decorator(wp):  # type: ignore[no-untyped-def]
             self.update(
                 (wp,),
                 ShouldBeInjectable(wp),
@@ -448,30 +458,53 @@ class Module(Broker, EventListener):
 
         return decorator(wrapped) if wrapped else decorator
 
+    def constant[**P, T](  # type: ignore[no-untyped-def]
+        self,
+        wrapped: Callable[P, T] | None = None,
+        /,
+        *,
+        on: TypeInfo[T] = (),
+        mode: Mode | ModeStr = Mode.get_default(),
+    ):
+        def decorator(wp):  # type: ignore[no-untyped-def]
+            instance = wp()
+            self.set_constant(
+                instance,
+                on=on,
+                mode=mode,
+            )
+            return wp
+
+        return decorator(wrapped) if wrapped else decorator
+
     def set_constant[T](
         self,
         instance: T,
         on: TypeInfo[T] = (),
         *,
+        alias: bool = False,
         mode: Mode | ModeStr = Mode.get_default(),
     ) -> Self:
-        cls = type(instance)
+        if not alias:
+            cls = type(instance)
+            on = (cls, on)
+
         self.injectable(
             lambda: instance,
             inject=False,
-            on=(cls, on),
+            on=on,
             mode=mode,
         )
         return self
 
-    def inject(
+    def inject[**P, T](  # type: ignore[no-untyped-def]
         self,
-        wrapped: Callable[..., Any] = None,
+        wrapped: Callable[P, T] | None = None,
         /,
         *,
         return_factory: bool = False,
     ):
-        def decorator(wp):
+        def decorator(wp):  # type: ignore[no-untyped-def]
             if not return_factory and isclass(wp):
                 wp.__init__ = self.inject(wp.__init__)
                 return wp
@@ -479,7 +512,7 @@ class Module(Broker, EventListener):
             function = InjectedFunction(wp)
 
             @function.on_setup
-            def listen():
+            def listen() -> None:
                 function.update(self)
                 self.add_listener(function)
 
@@ -487,11 +520,11 @@ class Module(Broker, EventListener):
 
         return decorator(wrapped) if wrapped else decorator
 
-    def find_instance[T](self, cls: type[T]) -> T:
+    def find_instance[T](self, cls: InputType[T]) -> T:
         injectable = self[cls]
         return injectable.get_instance()
 
-    def get_instance[T](self, cls: type[T]) -> T | None:
+    def get_instance[T](self, cls: InputType[T]) -> T | None:
         try:
             return self.find_instance(cls)
         except KeyError:
@@ -499,7 +532,7 @@ class Module(Broker, EventListener):
 
     def get_lazy_instance[T](
         self,
-        cls: type[T],
+        cls: InputType[T],
         *,
         cache: bool = False,
     ) -> Invertible[T | None]:
@@ -512,7 +545,7 @@ class Module(Broker, EventListener):
 
     def update[T](
         self,
-        classes: Iterable[type[T] | UnionType],
+        classes: Iterable[InputType[T]],
         injectable: Injectable[T],
         mode: Mode | ModeStr = Mode.get_default(),
     ) -> Self:
@@ -566,7 +599,7 @@ class Module(Broker, EventListener):
         module: Module,
         *,
         priority: Priority | PriorityStr = Priority.get_default(),
-    ):
+    ) -> Iterator[None]:
         self.use(module, priority=priority)
         yield
         self.stop_using(module)
@@ -580,6 +613,7 @@ class Module(Broker, EventListener):
 
         return self
 
+    @override
     @synchronized()
     def unlock(self) -> Self:
         for broker in self.__brokers:
@@ -599,12 +633,13 @@ class Module(Broker, EventListener):
         self.__channel.remove_listener(listener)
         return self
 
-    def on_event(self, event: Event, /) -> ContextManager:
+    @override
+    def on_event(self, event: Event, /) -> ContextManager[None] | None:
         self_event = ModuleEventProxy(self, event)
         return self.dispatch(self_event)
 
     @contextmanager
-    def dispatch(self, event: Event):
+    def dispatch(self, event: Event) -> Iterator[None]:
         self.__check_locking()
 
         with self.__channel.dispatch(event):
@@ -612,15 +647,15 @@ class Module(Broker, EventListener):
             message = str(event)
             self.__debug(message)
 
-    def __debug(self, message: object):
+    def __debug(self, message: object) -> None:
         for logger in tuple(self.__loggers):
             logger.debug(message)
 
-    def __check_locking(self):
+    def __check_locking(self) -> None:
         if self.is_locked:
             raise ModuleLockError(f"`{self}` is locked.")
 
-    def __move_module(self, module: Module, priority: Priority):
+    def __move_module(self, module: Module, priority: Priority) -> None:
         last = priority != Priority.HIGH
 
         try:
@@ -631,7 +666,7 @@ class Module(Broker, EventListener):
             ) from exc
 
     @classmethod
-    def from_name(cls, name: str) -> Self:
+    def from_name(cls, name: str) -> Module:
         with suppress(KeyError):
             return cls.__instances[name]
 
@@ -642,8 +677,8 @@ class Module(Broker, EventListener):
         return instance
 
     @classmethod
-    def default(cls) -> Self:
-        return cls.from_name("default")
+    def default(cls) -> Module:
+        return cls.from_name("__default__")
 
 
 """
@@ -653,7 +688,7 @@ InjectedFunction
 
 @dataclass(repr=False, frozen=True, slots=True)
 class Dependencies:
-    mapping: Mapping[str, Injectable]
+    mapping: Mapping[str, Injectable[Any]]
 
     def __bool__(self) -> bool:
         return bool(self.mapping)
@@ -674,7 +709,7 @@ class Dependencies:
         return OrderedDict(self)
 
     @classmethod
-    def from_mapping(cls, mapping: Mapping[str, Injectable]) -> Self:
+    def from_mapping(cls, mapping: Mapping[str, Injectable[Any]]) -> Self:
         return cls(mapping)
 
     @classmethod
@@ -682,7 +717,12 @@ class Dependencies:
         return cls.from_mapping({})
 
     @classmethod
-    def resolve(cls, signature: Signature, module: Module, owner: type = None) -> Self:
+    def resolve(
+        cls,
+        signature: Signature,
+        module: Module,
+        owner: type | None = None,
+    ) -> Self:
         dependencies = LazyMapping(cls.__resolver(signature, module, owner))
         return cls.from_mapping(dependencies)
 
@@ -691,11 +731,11 @@ class Dependencies:
         cls,
         signature: Signature,
         module: Module,
-        owner: type = None,
-    ) -> Iterator[tuple[str, Injectable]]:
+        owner: type | None = None,
+    ) -> Iterator[tuple[str, Injectable[Any]]]:
         for name, annotation in cls.__get_annotations(signature, owner):
             try:
-                injectable: Injectable = module[annotation]
+                injectable: Injectable[Any] = module[annotation]
             except KeyError:
                 continue
 
@@ -704,7 +744,7 @@ class Dependencies:
     @staticmethod
     def __get_annotations(
         signature: Signature,
-        owner: type = None,
+        owner: type | None = None,
     ) -> Iterator[tuple[str, type | Any]]:
         parameters = iter(signature.parameters.items())
 
@@ -721,56 +761,73 @@ class Arguments(NamedTuple):
     kwargs: Mapping[str, Any]
 
 
-class InjectedFunction(EventListener):
+class InjectedFunction[**P, T](EventListener):
     __slots__ = (
         "__dict__",
-        "__signature__",
         "__wrapped__",
         "__dependencies",
         "__owner",
         "__setup_queue",
     )
 
-    def __init__(self, wrapped: Callable[..., Any], /):
-        self.__update_vars_from(wrapped)
+    __signature__: Signature
+    __wrapped__: Callable[P, T]
+    __dependencies: Dependencies
+    __owner: type | None
+    __setup_queue: Queue[Callable[..., Any]] | None
+
+    def __init__(self, wrapped: Callable[P, T], /) -> None:
         update_wrapper(self, wrapped, updated=())
+        self.__update_vars_from(wrapped)
         self.__dependencies = Dependencies.empty()
         self.__owner = None
-        self.__setup_queue = Queue[Callable[..., Any]](maxsize=2)
-        self.on_setup(self.__set_signature)
+        self.__setup_queue = Queue()
 
+    @override
     def __repr__(self) -> str:  # pragma: no cover
         return repr(self.wrapped)
 
+    @override
     def __str__(self) -> str:  # pragma: no cover
         return str(self.wrapped)
 
-    def __call__(self, /, *args, **kwargs) -> Any:
+    def __call__(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
         self.__setup()
         arguments = self.bind(args, kwargs)
         return self.wrapped(*arguments.args, **arguments.kwargs)
 
-    def __get__(self, instance: object = None, owner: type = None) -> Self | MethodType:
+    def __get__(
+        self,
+        instance: object | None = None,
+        owner: type | None = None,
+    ) -> Self | MethodType:
         if instance is None:
             return self
 
         return MethodType(self, instance)
 
-    def __set_name__(self, owner: type, name: str):
+    def __set_name__(self, owner: type, name: str) -> None:
         self.set_owner(owner)
 
     @property
     def signature(self) -> Signature:
-        return self.__signature__
+        with suppress(AttributeError):
+            return self.__signature__
+
+        with synchronized():
+            signature = inspect.signature(self.wrapped, eval_str=True)
+            self.__signature__ = signature
+
+        return signature
 
     @property
-    def wrapped(self) -> Callable[..., Any]:
-        return self.__wrapped__  # type: ignore
+    def wrapped(self) -> Callable[P, T]:
+        return self.__wrapped__
 
     def bind(
         self,
         args: Iterable[Any] = (),
-        kwargs: Mapping[str, Any] = None,
+        kwargs: Mapping[str, Any] | None = None,
     ) -> Arguments:
         if kwargs is None:
             kwargs = {}
@@ -793,7 +850,7 @@ class InjectedFunction(EventListener):
         if self.__owner:
             raise TypeError("Function owner is already defined.")
 
-        self.__owner = owner  # type: ignore
+        self.__owner = owner
         return self
 
     @synchronized()
@@ -801,26 +858,42 @@ class InjectedFunction(EventListener):
         self.__dependencies = Dependencies.resolve(self.signature, module, self.__owner)
         return self
 
-    def on_setup(self, wrapped: Callable[..., Any] = None, /):
-        def decorator(wp):
-            self.__setup_queue.put_nowait(wp)
+    def on_setup[**_P, _T](self, wrapped: Callable[_P, _T] | None = None, /):  # type: ignore[no-untyped-def]
+        def decorator(wp):  # type: ignore[no-untyped-def]
+            queue = self.__setup_queue
+
+            if queue is None:
+                raise RuntimeError(f"`{self}` is already up.")
+
+            queue.put_nowait(wp)
             return wp
 
         return decorator(wrapped) if wrapped else decorator
 
     @singledispatchmethod
-    def on_event(self, event: Event, /) -> ContextManager | None:  # type: ignore
+    @override
+    def on_event(self, event: Event, /) -> ContextManager[None] | None:  # type: ignore[override]
         return None
 
     @on_event.register
     @contextmanager
-    def _(self, event: ModuleEvent, /):
+    def _(self, event: ModuleEvent, /) -> Iterator[None]:
         yield
         self.update(event.module)
 
-    def __setup(self):
+    @synchronized()
+    def __close_setup_queue(self) -> None:
+        if self.__setup_queue is None:
+            return
+
+        self.__setup_queue = None
+
+    def __setup(self) -> None:
         queue = self.__setup_queue
 
+        if queue is None:
+            return
+
         while True:
             try:
                 task = queue.get_nowait()
@@ -831,21 +904,18 @@ class InjectedFunction(EventListener):
             queue.task_done()
 
         queue.join()
+        self.__close_setup_queue()
 
-    def __set_signature(self) -> Self:
-        self.__signature__ = inspect.signature(self.wrapped, eval_str=True)
-        return self
-
-    def __update_vars_from(self, obj: Any):
+    def __update_vars_from(self, obj: Any) -> None:
         try:
             variables = vars(obj)
         except TypeError:
-            pass
+            ...
         else:
             self.__update_vars(variables)
 
-    def __update_vars(self, variables: Mapping[str, Any]):
-        restricted_vars = frozenset(
+    def __update_vars(self, variables: Mapping[str, Any]) -> None:
+        restricted_vars = frozenset(("__signature__", "__wrapped__")) | frozenset(
             var for var in dir(self) if not self.__is_dunder(var)
         )
         vars(self).update(
